@@ -3,7 +3,7 @@ package DBIx::OnlineDDL::Helper::MySQL;
 our $AUTHORITY = 'cpan:GSG';
 # ABSTRACT: Private OnlineDDL helper for MySQL-specific code
 use version;
-our $VERSION = 'v0.930.1'; # VERSION
+our $VERSION = 'v0.940.0'; # VERSION
 
 use v5.10;
 use Moo;
@@ -13,7 +13,8 @@ extends 'DBIx::OnlineDDL::Helper::Base';
 use Types::Standard qw( InstanceOf );
 
 use DBI::Const::GetInfoType;
-use Sub::Util qw( set_subname );
+use List::Util qw( first );
+use Sub::Util  qw( set_subname );
 
 use namespace::clean;  # don't export the above
 
@@ -157,15 +158,32 @@ sub rename_fks_in_table_sql {
     return $table_sql;
 }
 
-sub has_triggers_on_table {
+sub has_conflicting_triggers_on_table {
     my ($self, $table_name) = @_;
+    my $mmver = $self->mmver;
 
-    return $self->dbh_runner(run => set_subname '_has_triggers_on_table', sub {
-        $_->selectrow_array(
-            'SELECT trigger_name FROM information_schema.triggers WHERE event_object_schema = DATABASE() AND event_object_table = ?',
-            undef, $table_name
-        );
-    });
+    # Multiple triggers aren't allowed in MySQL 5.6
+    if ($mmver < 5.007) {
+        return $self->dbh_runner(run => set_subname '_has_triggers_on_table', sub {
+            $_->selectrow_array(
+                'SELECT trigger_name FROM information_schema.triggers WHERE event_object_schema = DATABASE() AND event_object_table = ?',
+                undef, $table_name
+            );
+        });
+    }
+    # MySQL 5.7+ allows them, so look for anything that looks like a leftover OnlineDDL
+    # trigger name.
+    else {
+        return $self->dbh_runner(run => set_subname '_has_onlineddl_triggers_on_table', sub {
+            $_->selectrow_array(
+                'SELECT trigger_name FROM information_schema.triggers WHERE '.join(' AND ',
+                    'event_object_schema = DATABASE()',
+                    'event_object_table = ?',
+                    'trigger_name LIKE ?',
+                ), undef, $table_name, "\%${table_name}\\_onlineddl\\_\%"
+            );
+        });
+    }
 }
 
 sub find_new_trigger_identifier {
@@ -174,7 +192,7 @@ sub find_new_trigger_identifier {
     return $self->find_new_identifier(
         $trigger_name => sub {
             $_[0]->selectrow_array(
-                'SELECT trigger_name FROM information_schema.triggers WHERE trigger_schema = DATABASE() AND trigger_name = ?',
+                'SELECT trigger_name FROM information_schema.triggers WHERE event_object_schema = DATABASE() AND trigger_name = ?',
                 undef, $_[1]
             );
         },
@@ -208,17 +226,17 @@ sub swap_tables {
 ### of databases/table/column combos.  Furthermore, the kludgy join to TABLE_CONSTRAINTS
 ### is entirely unnecessary.  See also: https://github.com/perl5-dbi/DBD-mysql/issues/326
 sub foreign_key_info {
-    my $self   = shift;
-    my $dbh    = $self->dbh;
-    my ($mver) = ($dbh->get_info($GetInfoType{SQL_DBMS_VER}) =~ /(\d+)\./);
+    my $self  = shift;
+    my $dbh   = $self->dbh;
+    my $mmver = $self->mmver;
 
     # MySQL 8's information_schema implementation isn't a complete dumpster fire of
     # in-memory jank, and totes won't send your precious DB server into a horrible
     # OOM death.  So, skip this noise in that case.
     #
     # More info: https://mysqlserverteam.com/mysql-8-0-improvements-to-information_schema/
-    return $dbh->foreign_key_info(@_) if $mver >= 8;
-    return if $mver < 5;  # not supported by OnlineDDL, anyway
+    return $dbh->foreign_key_info(@_) if $mmver >= 8;
+    return if $mmver < 5;  # not supported by OnlineDDL, anyway
 
     my (
         $pk_catalog, $pk_schema, $pk_table,
@@ -303,6 +321,98 @@ around 'rename_fks_back_to_original_stmts' => sub {
 
 # Keep Base->add_fks_back_to_child_tables_stmts (no DROPs on those)
 
+# Look for default FK-created indexes that get mysteriously renamed after the FKs are
+# recreated. (SM-3039)
+sub post_fk_add_cleanup_stmts {
+    my $self     = shift;
+    my $dbh      = $self->dbh;
+    my $mmver    = $self->mmver;
+    my $vars     = $self->vars;
+    my $catalog  = $vars->{catalog};
+    my $schema   = $vars->{schema};
+    my $idx_hash = $vars->{indexes}{definitions};
+
+    my @stmts;
+    foreach my $table_name (sort keys %$idx_hash) {
+        my %old_idx_data = %{ $idx_hash->{$table_name} };
+        my %new_idx_data = %{ $self->get_idx_hash($table_name) };
+
+        foreach my $index_name (sort keys %old_idx_data) {
+            next if $index_name eq 'PRIMARY';
+
+            my $old_idx = $old_idx_data{$index_name};
+            my $new_idx = $new_idx_data{$index_name};
+            my $old_col_str = join ', ', @{$old_idx->{columns}};
+
+            next if $new_idx &&
+                $old_col_str eq join(', ', @{$new_idx->{columns}}) &&
+                $old_idx->{unique} == $new_idx->{unique}
+            ;
+
+            # It failed one of the other checks?
+            if ($new_idx) {
+                my $conditional =
+                    $old_idx->{unique} != $new_idx->{unique} ?
+                    ($new_idx->{unique} ? "it is now UNIQUE" : "it is no longer UNIQUE") :
+                    "its columns have changed (".join(', ', @{$new_idx->{columns}}).")"
+                ;
+
+                $self->progress->message( join "\n",
+                    '',
+                    "WARNING: Found index $table_name.$index_name ($old_col_str), but $conditional!",
+                    "Please double-check that the indexes on the table are what you expect!",
+                    '',
+                );
+                next;
+            }
+
+            # It looks like we have a mismatch at this point.  Try to find the renamed index.
+            $new_idx = first {
+                $_->{name} ne 'PRIMARY' &&
+                $old_col_str eq join(', ', @{$_->{columns}}) &&
+                $old_idx->{unique} == $_->{unique}
+            } values %new_idx_data;
+
+            # It disappeared?
+            unless ($new_idx) {
+                $self->progress->message( join "\n",
+                    '',
+                    "WARNING: Found index $table_name.$index_name ($old_col_str), which may have been renamed, but a matching index cannot be found!",
+                    "Please double-check that the indexes on the table are what you expect!",
+                    '',
+                );
+                next;
+            }
+
+            my $new_index_name = $new_idx->{name};
+
+            # We can only rename in MySQL 5.7
+            if ($mmver < 5.007) {
+                $self->progress->message( join "\n",
+                    '',
+                    "WARNING: Found index $table_name.$index_name ($old_col_str), which was renamed to $new_index_name!",
+                    "The index cannot safely renamed in MySQL $mmver, so you may need to DROP/ADD this",
+                    "index a different way to correct the problem.",
+                    '',
+                );
+                next;
+            }
+
+            # Finally, add a RENAME INDEX statement
+            push @stmts, join(' ',
+                "ALTER TABLE",
+                $dbh->quote_identifier( $table_name ),
+                "RENAME INDEX",
+                $dbh->quote_identifier( $new_index_name ),
+                "TO",
+                $dbh->quote_identifier( $index_name ),
+            );
+        }
+    }
+
+    return @stmts;
+}
+
 1;
 
 __END__
@@ -317,7 +427,7 @@ DBIx::OnlineDDL::Helper::MySQL - Private OnlineDDL helper for MySQL-specific cod
 
 =head1 VERSION
 
-version v0.930.1
+version v0.940.0
 
 =head1 DESCRIPTION
 
