@@ -3,7 +3,7 @@ package DBIx::OnlineDDL::Helper::SQLite;
 our $AUTHORITY = 'cpan:GSG';
 # ABSTRACT: Private OnlineDDL helper for SQLite-specific code
 use version;
-our $VERSION = 'v0.930.1'; # VERSION
+our $VERSION = 'v0.940.0'; # VERSION
 
 use v5.10;
 use Moo;
@@ -12,7 +12,6 @@ extends 'DBIx::OnlineDDL::Helper::Base';
 
 use Types::Standard qw( InstanceOf );
 
-use DBI::Const::GetInfoType;
 use Sub::Util qw( set_subname );
 
 use namespace::clean;  # don't export the above
@@ -83,9 +82,7 @@ sub post_connection_stmts {
     # Also, while this change was introduced in 3.25.0, it seems to only manifest itself
     # when the driver reports version 3.26.0, possibly due to how their production
     # releases work.
-    no warnings 'numeric';
-    my $db_ver = $self->dbh->get_info( $GetInfoType{SQL_DBMS_VER} ) || 0;
-    push @stmts, 'PRAGMA legacy_alter_table = ON' if $db_ver+0 >= 3.26;
+    push @stmts, 'PRAGMA legacy_alter_table = ON' if $self->mmver >= 3.026;
 
     return @stmts;
 }
@@ -121,7 +118,7 @@ sub create_table_sql {
 
 # Keep Base->rename_fks_in_table_sql (not used)
 
-sub has_triggers_on_table {
+sub has_conflicting_triggers_on_table {
     my ($self, $table_name) = @_;
 
     return $self->dbh_runner(run => set_subname '_has_triggers_on_table', sub {
@@ -171,7 +168,7 @@ DBIx::OnlineDDL::Helper::SQLite - Private OnlineDDL helper for SQLite-specific c
 
 =head1 VERSION
 
-version v0.930.1
+version v0.940.0
 
 =head1 DESCRIPTION
 
